@@ -16,6 +16,7 @@ import os
 from threading import Thread
 from typing import (
     Dict,
+    Optional,
     Union,
 )
 
@@ -32,6 +33,7 @@ from requests.exceptions import (
 )
 
 from .builders import (
+    AsyncResult,
     build_namespaces,
     create_client_auth,
     create_environment,
@@ -39,6 +41,7 @@ from .builders import (
     validate_client_auth,
 )
 from .constants import DEFAULT_API_ROOT
+from .parallel import ParallelExecutionCollector
 from .queries import (
     ActivityMonitorQuery,
     GetMutationNameQuery,
@@ -117,6 +120,7 @@ class Qctrl:
 
         self.gql_api = self._build_client(email, password)
         self.gql_env = create_environment(self.gql_api)
+        self.collector: Optional[ParallelExecutionCollector] = None
         self._build_namespaces()
 
     @tenacity.retry(
@@ -229,14 +233,57 @@ class Qctrl:
         field_type = _get_mutation_result_type(self.gql_api.schema, mutation_name)
 
         refresh_query = self.gql_env.build_refresh_query(field_type)
-        response = refresh_query(action_id)
+        data = refresh_query(action_id)
 
-        result = self.gql_env.load_data(
-            field_type, response["coreAction"]["coreAction"]
-        )
-        self.gql_env.wait_for_completion(field_type, result, refresh_query)
+        result = self.gql_env.load_data(field_type, data)
+        self.gql_env.wait_for_completion(refresh_query, AsyncResult(field_type, result))
         return result
 
+    def parallel(self) -> ParallelExecutionCollector:
+        """Used for collecting multiple function calls to run in parallel.
+
+        Returns
+        -------
+        ParallelExecutionCollector
+            the collector object to be used as a context manager.
+        """
+        return ParallelExecutionCollector(self)
+
+    def is_collecting(self) -> bool:
+        """Checks if the object is in collection mode.
+
+        Returns
+        -------
+        bool
+            True if in collection mode, False otherwise.
+        """
+        return bool(self.collector)
+
+    def start_collection_mode(self, collector: ParallelExecutionCollector):
+        """Starts collection mode. All function calls will be collected
+        and executed when the collector object exits.
+
+        Parameters
+        ----------
+        collector: ParallelExecutionCollector
+            the collector object where function calls are stored.
+
+        Raises
+        ------
+        RuntimeError
+            unable to enter collection mode if already collecting.
+        """
+        if self.is_collecting():
+            raise RuntimeError("unable to nest parallel collections")
+
+        self.collector = collector
+
+    def stop_collection_mode(self):
+        """Stops collection mode. Function calls will be executed
+        immediately.
+        """
+        self.collector = None
+
     def _run_gql_query(self, query: str, variable_values: Dict = None) -> Dict:
         """
         Runs a GQL query in a Python script.
