@@ -1,4 +1,4 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
@@ -26,6 +26,8 @@ from gql.transport.exceptions import TransportServerError
 from graphql import (
     GraphQLField,
     GraphQLInputField,
+    GraphQLSchema,
+    GraphQLType,
 )
 from graphql.pyutils.undefined import UndefinedType
 from packaging import version
@@ -256,3 +258,33 @@ def _get_latest_qctrl_version() -> str:
     contents = requests.get("https://pypi.org/pypi/qctrl/json").json()
     latest_version = contents["info"]["version"]
     return latest_version
+
+
+def _get_mutation_result_type(schema: GraphQLSchema, mutation_name: str) -> GraphQLType:
+    """Returns the GraphQLType for the given mutation.
+
+    Parameters
+    ----------
+    mutation_name : str
+        The name of the mutation field in the schema.
+
+
+    Returns
+    -------
+    GraphQLType
+        Result type of the mutation
+
+    Raises
+    ------
+    KeyError
+        invalid mutation name.
+    """
+    mutation_type = schema.get_type("Mutation")
+    assert mutation_type
+
+    try:
+        mutation_field = mutation_type.fields[mutation_name]
+    except KeyError as error:
+        raise KeyError(f"unknown mutation: {mutation_name}") from error
+
+    return mutation_field.type
